@@ -8,6 +8,7 @@ from sqlalchemy import Integer
 from sqlalchemy import DateTime
 from sqlalchemy import Table
 from sqlalchemy import Unicode
+from sqlalchemy import func
 from sqlalchemy.sql.expression import or_
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm.exc import NoResultFound
@@ -146,7 +147,7 @@ SITE_ACL = [
     ['Allow', 'role:owner', ['view', 'add', 'edit', 'manage']],
     ]
 
-class PersistentACL(object):
+class PersistentACLMixin(object):
     """Manages access to ``self._acl`` which is a JSON- serialized
     representation of ``self.__acl__``.
     """
@@ -155,7 +156,7 @@ class PersistentACL(object):
     @staticmethod
     def _deserialize_ace(ace):
         ace = list(ace)
-        if ace[2] == PersistentACL.ALL_PERMISSIONS_SERIALIZED:
+        if ace[2] == PersistentACLMixin.ALL_PERMISSIONS_SERIALIZED:
             ace[2] = ALL_PERMISSIONS
         return tuple(ace)
 
@@ -163,7 +164,7 @@ class PersistentACL(object):
     def _serialize_ace(ace):
         ace = list(ace)
         if ace[2] == ALL_PERMISSIONS:
-            ace[2] = PersistentACL.ALL_PERMISSIONS_SERIALIZED
+            ace[2] = PersistentACLMixin.ALL_PERMISSIONS_SERIALIZED
         return ace
 
     def _get_acl(self):
@@ -197,10 +198,15 @@ def list_groups_raw(name, context):
     global or inherited groups are returned.
     """
     from kotti.resources import LocalGroup
-    session = DBSession()
-    return set(r[0] for r in session.query(LocalGroup.group_name).filter(
-        LocalGroup.node_id==context.id).filter(
-        LocalGroup.principal_name==name).all())
+    from kotti.resources import Node
+
+    if isinstance(context, Node):
+        return set(
+            r[0] for r in DBSession.query(LocalGroup.group_name).filter(
+            LocalGroup.node_id==context.id).filter(
+            LocalGroup.principal_name==name).all()
+            )
+    return set()
 
 def list_groups(name, context=None):
     """List groups for principal with a given ``name``.
@@ -385,7 +391,8 @@ class Principals(DictMixin):
         for key, value in kwargs.items():
             col = getattr(self.factory, key)
             if '*' in value:
-                filters.append(col.like(value.replace('*', '%')))
+                value = value.replace('*', '%').lower()
+                filters.append(func.lower(col).like(value))
             else:
                 filters.append(col == value)
 
