@@ -1,8 +1,11 @@
 from contextlib import contextmanager
 import os
 import unittest
+import warnings
 
+from mock import MagicMock
 import transaction
+from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy.exc import IntegrityError
 import colander
 from pyramid.authentication import CallbackAuthenticationPolicy
@@ -35,20 +38,16 @@ from kotti.security import map_principals_with_local_roles
 from kotti.security import get_principals
 from kotti.security import is_user
 from kotti.util import ViewLink
-from kotti.util import clear_request_cache
+from kotti.util import clear_cache
 
 BASE_URL = 'http://localhost:6543'
 
-## Unit tests
-
 class DummyRequest(testing.DummyRequest):
     is_xhr = False
 
     def is_response(self, ob):
-        if ( hasattr(ob, 'app_iter') and hasattr(ob, 'headerlist') and
-             hasattr(ob, 'status') ):
-            return True
-        return False
+        return ( hasattr(ob, 'app_iter') and hasattr(ob, 'headerlist') and
+                 hasattr(ob, 'status') )
 
 def testing_db_url():
     return os.environ.get('KOTTI_TEST_DB_STRING', 'sqlite://')
@@ -59,7 +58,8 @@ def _initTestingDB():
     session = initialize_sql(create_engine(database_url), drop_all=True)
     return session
 
-def setUp(**kwargs):
+def setUp(init_db=True, **kwargs):
+    #warnings.filterwarnings("error")
     tearDown()
     settings = conf_defaults.copy()
     settings['kotti.secret'] = 'secret'
@@ -68,10 +68,12 @@ def setUp(**kwargs):
     settings.update(kwargs.get('settings', {}))
     kwargs['settings'] = settings
     config = testing.setUp(**kwargs)
-
-    _initTestingDB()
     for name, renderer in DEFAULT_RENDERERS:
         config.add_renderer(name, renderer)
+
+    if init_db:
+        _initTestingDB()
+
     transaction.begin()
     return config
 
@@ -221,6 +223,28 @@ class TestNode(UnitTestBase):
         self.assertEquals(
             session.query(Node).filter(Node.name == u'subchild').count(), 0)
 
+        # We can pass a tuple as the key to more efficiently reach
+        # down to child objects:
+        root[u'child3'] = Node()
+        subchild33 = Node(name=u'subchild33', parent=root[u'child3'])
+        session.add(subchild33)
+        self.assertTrue(
+            root[u'child3', u'subchild33'] is root[u'child3'][u'subchild33'])
+        self.assertTrue(
+            root[(u'child3', u'subchild33')] is subchild33)
+        self.assertRaises(KeyError, root.__getitem__, (u'child3', u'bad-name'))
+        del root[u'child3']
+
+        # Overwriting an existing Node is an error; first delete manually!
+        child4 = Node(name=u'child4', parent=root)
+        session.add(child4)
+        self.assertEquals(root.keys(), [u'child4'])
+
+        child44 = Node(name=u'child4')
+        session.add(child44)
+        root[u'child4'] = child44
+        self.assertRaises(SQLAlchemyError, session.flush)
+        
     def test_node_copy(self):
         # Test some of Node's container methods:
         root = get_root()
@@ -239,6 +263,51 @@ class TestNode(UnitTestBase):
         root = get_root()
         self.assertRaises(ValueError, setattr, root, 'annotations', [])
 
+    def test_annotations_wrapper(self):
+        root = get_root()
+        root.__annotations__.people = {}
+        root.__annotations__.people.andy = {}
+        root.__annotations__.people.candy = {}
+        root.__annotations__.people.randy = {}
+        root.__annotations__.people.andy.age = 88
+        root.__annotations__.people.candy.age = 46
+        DBSession.flush()
+        DBSession.refresh(root)
+
+        self.assertTrue(root not in DBSession.dirty)
+        self.assertEqual(len(root.__annotations__.people), 3)
+        self.assertEqual(root.__annotations__.people.andy.age, 88)
+        self.assertEqual(root.__annotations__.people.candy.age, 46)
+        self.assertTrue(root not in DBSession.dirty)
+
+        root.__annotations__.people.andy.friends = ['randy']
+        self.assertTrue(root in DBSession.dirty)
+        DBSession.flush()
+        DBSession.refresh(root)
+        self.assertEqual(
+            root.__annotations__.people.andy.friends, ['randy'])
+        self.assertEqual(
+            root.__annotations__['people'].andy['friends'], ['randy'])
+
+        root.__annotations__.people.candy.age = 77
+        self.assertTrue(root in DBSession.dirty)
+        DBSession.flush()
+        DBSession.refresh(root)
+            
+        del root.__annotations__.people.candy
+        self.assertTrue(root in DBSession.dirty)
+        DBSession.flush()
+        DBSession.refresh(root)
+        self.assertEquals(sorted(root.__annotations__.people.keys()),
+                          ['andy', 'randy'])
+
+        self.assertRaises(
+            AttributeError,
+            getattr, root.__annotations__.people.randy, 'age')
+        self.assertRaises(
+            KeyError,
+            root.__annotations__.people.randy.__getitem__, 'age')
+
 class TestSecurity(UnitTestBase):
     def test_root_default(self):
         root = get_root()
@@ -257,6 +326,9 @@ class TestSecurity(UnitTestBase):
         self.assertEqual(
             list_groups_raw(u'bob', root), set(['role:editor']))
 
+    def test_not_a_node(self):
+        self.assertEqual(list_groups_raw(u'bob', object()), set())
+
     def test_overwrite_and_delete(self):
         root = get_root()
         set_groups('bob', root, ['role:editor'])
@@ -750,7 +822,7 @@ class TestEvents(UnitTestBase):
         self.assertEqual(child.owner, u'bob')
         self.assertEqual(list_groups(u'bob', child), [u'role:owner'])
 
-        clear_request_cache()
+        clear_cache()
         # The event listener does not set the role again for subitems:
         grandchild = child[u'grandchild'] = Content()
         session.flush()
@@ -758,6 +830,46 @@ class TestEvents(UnitTestBase):
         self.assertEqual(list_groups(u'bob', grandchild), [u'role:owner'])
         self.assertEqual(len(list_groups_raw(u'bob', grandchild)), 0)
 
+    def test_sqlalchemy_events(self):
+        from kotti import events
+
+        insert_events = []
+        def insert(event):
+            insert_events.append(event)
+        update_events = []
+        def update(event):
+            update_events.append(event)
+        delete_events = []
+        def delete(event):
+            delete_events.append(event)
+
+        lis = events.objectevent_listeners
+        lis[(events.ObjectInsert, None)].append(insert)
+        lis[(events.ObjectUpdate, None)].append(update)
+        lis[(events.ObjectDelete, None)].append(delete)
+
+        root = get_root()
+        child = root[u'child'] = Content()
+        DBSession.flush()
+        self.assertEqual(
+            (len(insert_events), len(update_events), len(delete_events)),
+            (1, 0, 0))
+        self.assertEqual(insert_events[0].object, child)
+
+        child.title = u"Bar"
+        DBSession.flush()
+        self.assertEqual(
+            (len(insert_events), len(update_events), len(delete_events)),
+            (1, 1, 0))
+        self.assertEqual(update_events[0].object, child)
+
+        DBSession.delete(child)
+        DBSession.flush()
+        self.assertEqual(
+            (len(insert_events), len(update_events), len(delete_events)),
+            (1, 1, 1))
+        self.assertEqual(delete_events[0].object, child)
+
 @contextmanager
 def contents_addable():
     # Allow Nodes to be added to documents:
@@ -1217,7 +1329,8 @@ class TestTemplateAPI(UnitTestBase):
         register(RenderAboveContent, None, render_something)
 
         api = self._make()
-        self.assertEqual(api.slots['abovecontent'], [u'Hello, My Site!'])
+        self.assertEqual(api.slots.abovecontent,
+                         [u'Hello, %s!' % api.context.title])
 
         # Slot renderers may also return lists:
         def render_a_list(context, request):
@@ -1225,19 +1338,31 @@ class TestTemplateAPI(UnitTestBase):
         register(RenderAboveContent, None, render_a_list)
         api = self._make()
         self.assertEqual(
-            api.slots['abovecontent'],
-            [u'Hello, My Site!', u'a', u'list']
+            api.slots.abovecontent,
+            [u'Hello, %s!' % api.context.title, u'a', u'list']
             )
 
-    def test_slots_empty(self):
-        api = self._make()
-        self.assertEqual(
-            sorted(api.slots.keys()),
-            ['abovecontent', 'beforebodyend', 'belowcontent', 'inhead',
-             'left', 'right']
+        self.assertRaises(
+            AttributeError,
+            getattr, api.slots, 'foobar'
             )
-        for key in api.slots.keys():
-            self.assertEqual(api.slots[key], [])
+
+    def test_slots_only_rendered_when_accessed(self):
+        from kotti.views.slots import register, RenderAboveContent
+
+        called = []
+        def render_something(context, request):
+            called.append(True)
+        register(RenderAboveContent, None, render_something)
+        
+        api = self._make()
+        api.slots.belowcontent
+        self.assertFalse(called)
+
+        api.slots.abovecontent
+        self.assertEquals(len(called), 1)
+        api.slots.abovecontent
+        self.assertEquals(len(called), 1)
 
     def test_slots_render_local_navigation(self):
         from kotti.views.slots import render_local_navigation
@@ -1315,6 +1440,13 @@ class TestTemplateAPI(UnitTestBase):
         self.assertEqual(
             api.render_view(context=api.context, request=api.request), u'first')
 
+    def test_render_template(self):
+        renderer = MagicMock()
+        self.config.testing_add_renderer('my-rendererer', renderer)
+        api = self._make()
+        api.render_template('my-rendererer', some='variable')
+        self.assertEqual(renderer.call_args[0][0], {'some': 'variable'})
+
     def test_get_type(self):
         from kotti.resources import Document
         api = self._make()
@@ -1371,16 +1503,18 @@ class TestUtil(UnitTestBase):
 
 class TestRequestCache(UnitTestBase):
     def setUp(self):
+        from kotti.util import request_cache
+
         registry = Registry('testing')
         request = DummyRequest()
         request.registry = registry
         super(TestRequestCache, self).setUp(registry=registry, request=request)
+        self.cache_decorator = request_cache
 
     def test_it(self):
-        from kotti.util import request_cache
         called = []
 
-        @request_cache(lambda a, b: (a, b))
+        @self.cache_decorator(lambda a, b: (a, b))
         def my_fun(a, b):
             called.append((a, b))
 
@@ -1390,19 +1524,18 @@ class TestRequestCache(UnitTestBase):
         my_fun(2, 1)
         self.assertEqual(len(called), 2)
 
-        clear_request_cache()
+        clear_cache()
         my_fun(1, 2)
         self.assertEqual(len(called), 3)
 
     def test_dont_cache(self):
-        from kotti.util import request_cache
         from kotti.util import DontCache
         called = []
 
         def dont_cache(a, b):
             raise DontCache
 
-        @request_cache(dont_cache)
+        @self.cache_decorator(dont_cache)
         def my_fun(a, b):
             called.append((a, b))
 
@@ -1410,7 +1543,12 @@ class TestRequestCache(UnitTestBase):
         my_fun(1, 2)
         self.assertEqual(len(called), 2)
 
-## Functional tests
+class TestLRUCache(TestRequestCache):
+    def setUp(self):
+        from kotti.util import lru_cache
+        
+        super(TestLRUCache, self).setUp()
+        self.cache_decorator = lru_cache
 
 def setUpFunctional(global_config=None, **settings):
     import wsgi_intercept.zope_testbrowser
