@@ -1,7 +1,9 @@
+from UserDict import DictMixin
 from pyramid.compat import json
 import string
 import urllib
 
+from repoze.lru import LRUCache
 from pyramid.threadlocal import get_current_request
 from pyramid.url import resource_url
 from sqlalchemy.types import TypeDecorator, VARCHAR
@@ -42,6 +44,47 @@ class MutationDict(Mutable, dict):
         dict.__delitem__(self, key)
         self.changed()
 
+def _keyerror_to_attributeerror(func):
+    def decorator(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except KeyError as e:
+            raise AttributeError(*e.args)
+    return decorator
+
+class MutableAnnotationsWrapper(object, DictMixin):
+    def __init__(self, data, mutable):
+        self.__dict__['__data__'] = data
+        self.__dict__['__mutable__'] = mutable
+
+    def changed(self):
+        self.__mutable__.changed()
+
+    def __getitem__(self, key):
+        value = self.__data__[key]
+        if isinstance(value, dict):
+            value = self.__class__(value, self.__mutable__)
+        return value
+    __getattr__ = _keyerror_to_attributeerror(__getitem__)
+
+    def __setitem__(self, key, value):
+        self.__data__[key] = value
+        self.changed()
+    __setattr__ = _keyerror_to_attributeerror(__setitem__)
+
+    def __delitem__(self, key):
+        del self.__data__[key]
+        self.changed()
+    __delattr__ = _keyerror_to_attributeerror(__delitem__)
+
+    def keys(self):
+        return self.__data__.keys()
+
+class MutableAnnotationsMixin(object):
+    @property
+    def __annotations__(self):
+        return MutableAnnotationsWrapper(self.annotations, self.annotations)
+
 class ViewLink(object):
     def __init__(self, path, title=None):
         self.path = path
@@ -66,22 +109,28 @@ class ViewLink(object):
     def __repr__(self):
         return "ViewLink(%r, %r)" % (self.path, self.title)
 
-_CACHE_ATTR = 'kotti_cache'
-
 class DontCache(Exception):
     pass
 
-def request_cache(compute_key):
+_CACHE_ATTR = 'kotti_cache'
+
+def request_container():
+    request = get_current_request()
+    if request is None:
+        return None
+    cache = getattr(request, _CACHE_ATTR, None)
+    if cache is None:
+        cache = {}
+        setattr(request, _CACHE_ATTR, cache)
+    return cache
+
+def cache(compute_key, container_factory):
     marker = object()
     def decorator(func):
         def replacement(*args, **kwargs):
-            request = get_current_request()
-            if request is None:
-                return func(*args, **kwargs)
-            cache = getattr(request, _CACHE_ATTR, None)
+            cache = container_factory()
             if cache is None:
-                cache = {}
-                setattr(request, _CACHE_ATTR, cache)
+                return func(*args, **kwargs)
             try:
                 key = compute_key(*args, **kwargs)
             except DontCache:
@@ -99,10 +148,22 @@ def request_cache(compute_key):
         return replacement
     return decorator
 
-def clear_request_cache(): # only useful for tests really
+def request_cache(compute_key):
+    return cache(compute_key, request_container)
+
+class LRUCacheSetItem(LRUCache):
+    __setitem__ = LRUCache.put
+
+_lru_cache = LRUCacheSetItem(1000)
+
+def lru_cache(compute_key):
+    return cache(compute_key, lambda: _lru_cache)
+
+def clear_cache(): # only useful for tests really
     request = get_current_request()
     if request is not None:
         setattr(request, _CACHE_ATTR, None)
+    _lru_cache.clear()
 
 def extract_from_settings(prefix):
     """
