@@ -1,3 +1,5 @@
+from datetime import datetime
+
 import colander
 from deform import Form
 from deform import ValidationFailure
@@ -14,25 +16,25 @@ from kotti.message import send_set_password
 from kotti.resources import get_root
 from kotti.resources import Node
 from kotti.security import get_principals
-from kotti.views.util import TemplateAPIEdit
+from kotti.views.util import template_api
 
 def _find_user(login):
     principals = get_principals()
     principal = principals.get(login)
-    if principal is None:
+    if principal is not None:
+        return principal
+    else:
         try:
             Email().to_python(login)
         except Exception:
             pass
         else:
-            results = [r for r in principals.search(email=login)]
-            if results:
-                principal = results[0]
-    return principal
+            for p in principals.search(email=login):
+                return p
 
 def login(context, request):
     root = get_root(request)
-    api = TemplateAPIEdit(root, request)
+    api = template_api(root, request)
     principals = get_principals()
 
     came_from = request.params.get('came_from', request.url)
@@ -44,10 +46,11 @@ def login(context, request):
         user = _find_user(login)
 
         if (user is not None and user.active and 
-            user.password == principals.hash_password(password)):
+            principals.validate_password(password, user.password)):
             headers = remember(request, login)
             request.session.flash(
                 u"Welcome, %s!" % user.title or user.name, 'success')
+            user.last_login_date = datetime.now()
             return HTTPFound(location=came_from, headers=headers)
         request.session.flash(u"Login failed.", 'error')
 
@@ -106,25 +109,25 @@ def set_password(context, request,
         else:
             token = appstruct['token']
             email = appstruct['email']
-            principals = get_principals()
-            results = principals.search(email=email)
-            if results:
-                user = results[0]
-                if validate_token(user, token) and token == user.confirm_token:
-                    password = appstruct['password']
-                    user.password = principals.hash_password(password)
-                    user.confirm_token = None
-                    headers = remember(request, user.name)
-                    location = resource_url(context, request)
-                    request.session.flash(success_msg, 'success')
-                    return HTTPFound(location=location, headers=headers)
-            request.session.flash(
-                u"Your password reset token may have expired.", 'error')
+            user = _find_user(email)
+            if (user is not None and
+                validate_token(user, token) and
+                token == user.confirm_token):
+                password = appstruct['password']
+                user.password = get_principals().hash_password(password)
+                user.confirm_token = None
+                headers = remember(request, user.name)
+                location = resource_url(context, request)
+                request.session.flash(success_msg, 'success')
+                return HTTPFound(location=location, headers=headers)
+            else:
+                request.session.flash(
+                    u"Your password reset token may have expired.", 'error')
 
     if rendered_form is None:
         rendered_form = form.render(request.params.items())
 
-    api = TemplateAPIEdit(
+    api = template_api(
         context, request,
         page_title=u"Reset your password - %s" % context.title,
         )
