@@ -6,7 +6,6 @@ from urllib import urlencode
 
 from pyramid.httpexceptions import HTTPFound
 from pyramid.exceptions import Forbidden
-from pyramid.view import is_response
 import colander
 import deform
 from deform import Form
@@ -26,7 +25,7 @@ from kotti.security import list_groups_raw
 from kotti.security import list_groups_ext
 from kotti.security import set_groups
 from kotti.views.site_setup import CONTROL_PANEL_LINKS
-from kotti.views.util import TemplateAPIEdit
+from kotti.views.util import template_api
 from kotti.views.util import is_root
 from kotti.views.util import FormController
 
@@ -39,7 +38,7 @@ def roles_form_handler(context, request, available_role_names, groups_lister):
             if name.startswith('orig-role::'):
                 # orig-role::* is hidden checkboxes that allow us to
                 # see what checkboxes were in the form originally
-                token, principal_name, role_name = name.split('::')
+                token, principal_name, role_name = unicode(name).split(u'::')
                 if role_name not in available_role_names:
                     raise Forbidden()
                 new_value = bool(request.params.get(
@@ -51,7 +50,8 @@ def roles_form_handler(context, request, available_role_names, groups_lister):
 
         for principal_name, new_role_names in p_to_r.items():
             # We have to be careful with roles that aren't mutable here:
-            orig_role_names = set(groups_lister(principal_name, context))
+            orig_role_names = set(
+                groups_lister(principal_name, context))
             orig_sharing_role_names = set(
                 r for r in orig_role_names if r in available_role_names)
             if new_role_names != orig_sharing_role_names:
@@ -113,7 +113,6 @@ def share_node(context, request):
     available_roles = [ROLES[role_name] for role_name in SHARING_ROLES]
 
     return {
-        'api': TemplateAPIEdit(context, request),
         'entries': entries,
         'available_roles': available_roles,
         }
@@ -256,11 +255,12 @@ def _massage_groups_out(appstruct):
     return d
 
 def users_manage(context, request):
-    api = TemplateAPIEdit(
+    api = template_api(
         context, request,
-        page_title=u"User Management - %s" % context.title,
         cp_links=CONTROL_PANEL_LINKS,
         )
+    api.page_title = u"User Management - %s" % api.site_title
+
     principals = get_principals()
 
     def groups_lister(principal_name, context):
@@ -325,7 +325,7 @@ def users_manage(context, request):
         post_key='add-user',
         )
     user_addform = user_fc(context, request)
-    if is_response(user_addform):
+    if request.is_response(user_addform):
         return user_addform
 
     gschema = group_schema()
@@ -341,7 +341,7 @@ def users_manage(context, request):
         post_key='add-group',
         )
     group_addform = group_fc(context, request)
-    if is_response(group_addform):
+    if request.is_response(group_addform):
         return group_addform
 
     return {
@@ -356,7 +356,7 @@ def user_manage(context, request):
     username = request.params['name']
     principal = get_principals()[username]
 
-    api = TemplateAPIEdit(
+    api = template_api(
         context, request,
         page_title=u"Edit User - %s" % context.title,
         cp_links=CONTROL_PANEL_LINKS,
@@ -383,7 +383,7 @@ def user_manage(context, request):
         edit_item=edit_principal,
         )
     form = user_fc(principal, request)
-    if is_response(form):
+    if request.is_response(form):
         return form
 
     return {
@@ -392,17 +392,14 @@ def user_manage(context, request):
         }
 
 def preferences(context, request):
-    api = TemplateAPIEdit(
-        context, request,
-        page_title=u"My preferences - %s" % context.title,
-        )
-
+    api = template_api(context, request)
+    api.page_title=u"My preferences - %s" % api.site_title
     user = api.user
     uschema = user_schema(PrincipalBasic())
     user_form = Form(uschema, buttons=('save', 'cancel'))
     user_fc = FormController(user_form)
     form = user_fc(user, request)
-    if is_response(form):
+    if request.is_response(form):
         return form
 
     return {
